;///////////////////////////////////////////////////////////////////////////
;//
;//  Copyright (C) Microsoft Corporation.  All Rights Reserved.
;//
;//  File:       d3dx9core.inc
;//  Content:    D3DX core types and functions
;//
;//  Translated to .ASM by: Siekmanski
;//
;///////////////////////////////////////////////////////////////////////////

ifndef __D3DX9CORE_H__
__D3DX9CORE_H__ equ 1

;///////////////////////////////////////////////////////////////////////////
;// D3DX_SDK_VERSION:
;// -----------------
;// This identifier is passed to D3DXCheckVersion in order to ensure that an
;// application was built against the correct header files and lib files. 
;// This number is incremented whenever a header (or other) change would 
;// require applications to be rebuilt. If the version doesn't match, 
;// D3DXCreateVersion will return FALSE. (The number itself has no meaning.)
;///////////////////////////////////////////////////////////////////////////

D3DX_VERSION		equ 0901h
D3DX_SDK_VERSION	equ 21

D3DXCheckVersion	PROTO :DWORD,:DWORD

;///////////////////////////////////////////////////////////////////////////
;// D3DXGetDriverLevel:
;//    Returns driver version information:
;//
;//    700 - DX7 level driver
;//    800 - DX8 level driver
;//    900 - DX9 level driver
;///////////////////////////////////////////////////////////////////////////

D3DXGetDriverLevel	PROTO :DWORD

;///////////////////////////////////////////////////////////////////////////
;// ID3DXBuffer:
;// ------------
;// The buffer object is used by D3DX to return arbitrary size data.
;//
;// GetBufferPointer -
;//    Returns a pointer to the beginning of the buffer.
;//
;// GetBufferSize -
;//    Returns the size of the buffer, in bytes.
;///////////////////////////////////////////////////////////////////////////

;// {932E6A7E-C68E-45dd-A7BF-53D19C86DB1F}
IID_ID3DXBuffer	TEXTEQU <{0932e6a7eh,0c68eh,045ddh,{0a7h,0bfh,053h,0d1h,09ch,086h,0dbh,01fh}}>

_vtID3DXBuffer MACRO CastName:REQ
	_vtIUnknown CastName
	&CastName&_GetBufferPointer	comethod1 ?
	&CastName&_GetBufferSize	comethod1 ?
ENDM

ID3DXBuffer struct
	_vtID3DXBuffer	ID3DXBuffer
ID3DXBuffer ends

LPD3DXBUFFER		typedef PTR DWORD

;//////////////////////////////////////////////////////////////////////////////
;// D3DXSPRITE flags:
;// -----------------
;// D3DXSPRITE_DONOTSAVESTATE
;//   Specifies device state is not to be saved and restored in Begin/End.
;// D3DXSPRITE_DONOTMODIFY_RENDERSTATE
;//   Specifies device render state is not to be changed in Begin.  The device
;//   is assumed to be in a valid state to draw vertices containing POSITION0, 
;//   TEXCOORD0, and COLOR0 data.
;// D3DXSPRITE_OBJECTSPACE
;//   The WORLD, VIEW, and PROJECTION transforms are NOT modified.  The 
;//   transforms currently set to the device are used to transform the sprites 
;//   when the batch is drawn (at Flush or End).  If this is not specified, 
;//   WORLD, VIEW, and PROJECTION transforms are modified so that sprites are 
;//   drawn in screenspace coordinates.
;// D3DXSPRITE_BILLBOARD
;//   Rotates each sprite about its center so that it is facing the viewer.
;// D3DXSPRITE_ALPHABLEND
;//   Enables ALPHABLEND(SRCALPHA, INVSRCALPHA) and ALPHATEST(alpha > 0).
;//   ID3DXFont expects this to be set when drawing text.
;// D3DXSPRITE_SORT_TEXTURE
;//   Sprites are sorted by texture prior to drawing.  This is recommended when
;//   drawing non-overlapping sprites of uniform depth.  For example, drawing
;//   screen-aligned text with ID3DXFont.
;// D3DXSPRITE_SORT_DEPTH_FRONTTOBACK
;//   Sprites are sorted by depth front-to-back prior to drawing.  This is 
;//   recommended when drawing opaque sprites of varying depths.
;// D3DXSPRITE_SORT_DEPTH_BACKTOFRONT
;//   Sprites are sorted by depth back-to-front prior to drawing.  This is 
;//   recommended when drawing transparent sprites of varying depths.
;//////////////////////////////////////////////////////////////////////////////

D3DXSPRITE_DONOTSAVESTATE		equ (1 shl 0)
D3DXSPRITE_DONOTMODIFY_RENDERSTATE	equ (1 shl 1)
D3DXSPRITE_OBJECTSPACE			equ (1 shl 2)
D3DXSPRITE_BILLBOARD			equ (1 shl 3)
D3DXSPRITE_ALPHABLEND			equ (1 shl 4)
D3DXSPRITE_SORT_TEXTURE			equ (1 shl 5)
D3DXSPRITE_SORT_DEPTH_FRONTTOBACK	equ (1 shl 6)
D3DXSPRITE_SORT_DEPTH_BACKTOFRONT	equ (1 shl 7)

;//////////////////////////////////////////////////////////////////////////////
;// ID3DXSprite:
;// ------------
;// This object intends to provide an easy way to drawing sprites using D3D.
;//
;// Begin - 
;//    Prepares device for drawing sprites.
;//
;// Draw -
;//    Draws a sprite.  Before transformation, the sprite is the size of 
;//    SrcRect, with its top-left corner specified by Position.  The color 
;//    and alpha channels are modulated by Color.
;//
;// Flush -
;//    Forces all batched sprites to submitted to the device.
;//
;// End - 
;//    Restores device state to how it was when Begin was called.
;//
;// OnLostDevice, OnResetDevice -
;//    Call OnLostDevice() on this object before calling Reset() on the
;//    device, so that this object can release any stateblocks and video
;//    memory resources.  After Reset(), the call OnResetDevice().
;//////////////////////////////////////////////////////////////////////////////

;// {D4715B38-6C44-472a-9024-6E2B0321CAC6}
IID_ID3DXSprite	TEXTEQU <{0d4715b38h,06c44h,0472ah,{090h,024h,06eh,02bh,003h,021h,0cah,0c6h}}>

_vtID3DXSprite MACRO CastName:REQ
	_vtIUnknown CastName
	&CastName&_GetDevice		comethod2 ?
	&CastName&_GetTransform		comethod2 ?
	&CastName&_SetTransform		comethod2 ?
	&CastName&_SetWorldViewRH	comethod3 ?
	&CastName&_SetWorldViewLH	comethod3 ?
	&CastName&_Begin		comethod2 ?
	&CastName&_Draw			comethod6 ?
	&CastName&_Flush		comethod1 ?
	&CastName&_End			comethod1 ?
	&CastName&_OnLostDevice		comethod1 ?
	&CastName&_OnResetDevice	comethod1 ?
ENDM

ID3DXSprite struct
	_vtID3DXSprite	ID3DXSprite
ID3DXSprite ends

LPD3DXSPRITE		typedef PTR DWORD

D3DXCreateSprite	PROTO :DWORD,:DWORD

;//////////////////////////////////////////////////////////////////////////////
;// ID3DXFont:
;// ----------
;// Font objects contain the textures and resources needed to render a specific 
;// font on a specific device.
;//
;// GetGlyphData -
;//    Returns glyph cache data, for a given glyph.
;//
;// PreloadCharacters/PreloadGlyphs/PreloadText -
;//    Preloads glyphs into the glyph cache textures.
;//
;// DrawText -
;//    Draws formatted text on a D3D device.  Some parameters are 
;//    surprisingly similar to those of GDI's DrawText function.  See GDI 
;//    documentation for a detailed description of these parameters.
;//    If pSprite is NULL, an internal sprite object will be used.
;//
;// OnLostDevice, OnResetDevice -
;//    Call OnLostDevice() on this object before calling Reset() on the
;//    device, so that this object can release any stateblocks and video
;//    memory resources.  After Reset(), the call OnResetDevice().
;//////////////////////////////////////////////////////////////////////////////

_D3DXFONT_DESCA struct
	dx_Height	UINT ?
	dx_Width	UINT ?
	Weight		UINT ?
	MipLevels	UINT ?
	Italic		BOOL ?
	CharSet		BYTE ?
	OutputPrecision	BYTE ?
	Quality		BYTE ?
	PitchAndFamily	BYTE ?
	FaceName	CHAR LF_FACESIZE dup (?)
_D3DXFONT_DESCA ends

D3DXFONT_DESCA		typedef _D3DXFONT_DESCA
LPD3DXFONT_DESCA	typedef PTR _D3DXFONT_DESCA

_D3DXFONT_DESCW struct
	dx_Height	UINT ?
	dx_Width	UINT ?
	Weight		UINT ?
	MipLevels	UINT ?
	Italic		BOOL ?
	CharSet		BYTE ?
	OutputPrecision	BYTE ?
	Quality		BYTE ?
	PitchAndFamily	BYTE ?
	FaceName	WCHAR LF_FACESIZE dup (?)
_D3DXFONT_DESCW ends

D3DXFONT_DESCW		typedef _D3DXFONT_DESCW
LPD3DXFONT_DESCW	typedef PTR _D3DXFONT_DESCW

ifdef UNICODE
D3DXFONT_DESC		typedef D3DXFONT_DESCW
LPD3DXFONT_DESC		typedef LPD3DXFONT_DESCW
else
D3DXFONT_DESC		typedef D3DXFONT_DESCA
LPD3DXFONT_DESC		typedef LPD3DXFONT_DESCA 
endif

;// {0B8D1536-9EEC-49b0-A5AD-93CF63AFB7C6}
IID_ID3DXFont		TEXTEQU <{00b8d1536h,09eech,049b0h,{0a5h,0adh,093h,0cfh,063h,0afh,0b7h,0c6h}}>

_vtID3DXFont MACRO CastName:REQ
	_vtIUnknown CastName
	&CastName&_GetDevice		comethod2 ?
	&CastName&_GetDescA		comethod2 ?
	&CastName&_GetDescW		comethod2 ?
	&CastName&_GetDC		comethod1 ?
	&CastName&_GetGlyphData		comethod5 ?
	&CastName&_PreloadCharacters	comethod3 ?
	&CastName&_PreloadGlyphs	comethod3 ?
	&CastName&_PreloadTextA		comethod3 ?
	&CastName&_PreloadTextW		comethod3 ?
	&CastName&_DrawTextA		comethod7 ?
	&CastName&_DrawTextW		comethod7 ?
	&CastName&_OnLostDevice		comethod1 ?
	&CastName&_OnResetDevice	comethod1 ?
	&CastName&_GetDesc		comethod2 ?
	&CastName&_PreloadText		comethod3 ?
ENDM

ID3DXFont struct
	_vtID3DXFont	ID3DXFont
ID3DXFont ends

LPD3DXFONT		typedef PTR DWORD

ifndef DrawText
ifdef UNICODE
DrawText		equ DrawTextW
else
DrawText		equ DrawTextA
endif
endif

D3DXCreateFontA		PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD
D3DXCreateFontW		PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD

ifdef UNICODE
D3DXCreateFont		equ D3DXCreateFontA
else
D3DXCreateFont		equ D3DXCreateFontA
endif

D3DXCreateFontIndirectA	PROTO :DWORD,:DWORD,:DWORD
D3DXCreateFontIndirectW	PROTO :DWORD,:DWORD,:DWORD

ifdef UNICODE
D3DXCreateFontIndirect	equ D3DXCreateFontIndirectW
else
D3DXCreateFontIndirect	equ D3DXCreateFontIndirectA
endif

;///////////////////////////////////////////////////////////////////////////
;// ID3DXRenderToSurface:
;// ---------------------
;// This object abstracts rendering to surfaces.  These surfaces do not 
;// necessarily need to be render targets.  If they are not, a compatible
;// render target is used, and the result copied into surface at end scene.
;//
;// BeginScene, EndScene -
;//    Call BeginScene() and EndScene() at the beginning and ending of your
;//    scene.  These calls will setup and restore render targets, viewports, 
;//    etc.. 
;//
;// OnLostDevice, OnResetDevice -
;//    Call OnLostDevice() on this object before calling Reset() on the
;//    device, so that this object can release any stateblocks and video
;//    memory resources.  After Reset(), the call OnResetDevice().
;///////////////////////////////////////////////////////////////////////////;

_D3DXRTS_DESC struct
	dx_Width		UINT ?
	dx_Height		UINT ?
	Format			D3DFORMAT ?
	DepthStencil		BOOL ?
	DepthStencilFormat	D3DFORMAT ?
_D3DXRTS_DESC ends

D3DXRTS_DESC		typedef _D3DXRTS_DESC
LPD3DXRTS_DESC		typedef PTR _D3DXRTS_DESC

;// {0D014791-8863-4c2c-A1C0-02F3E0C0B653}
IID_ID3DXRenderToSurface	TEXTEQU <{00d014791h,08863h,04c2ch,{0a1h,0c0h,002h,0f3h,0e0h,0c0h,0b6h,053h}}>

_vtID3DXRenderToSurface MACRO CastName:REQ
	_vtIUnknown CastName
	&CastName&_GetDevice		comethod2 ?
	&CastName&_GetDesc		comethod2 ?
	&CastName&_BeginScene		comethod3 ?
	&CastName&_EndScene		comethod2 ?
	&CastName&_OnLostDevice		comethod1 ?
	&CastName&_OnResetDevice	comethod1 ?
ENDM

ID3DXRenderToSurface struct
	_vtID3DXRenderToSurface	ID3DXRenderToSurface
ID3DXRenderToSurface ends

LPD3DXRENDERTOSURFACE		typedef PTR DWORD

D3DXCreateRenderToSurface	PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD

;///////////////////////////////////////////////////////////////////////////
;// ID3DXRenderToEnvMap:
;// --------------------
;// This object abstracts rendering to environment maps.  These surfaces 
;// do not necessarily need to be render targets.  If they are not, a 
;// compatible render target is used, and the result copied into the
;// environment map at end scene.
;//
;// BeginCube, BeginSphere, BeginHemisphere, BeginParabolic -
;//    This function initiates the rendering of the environment map.  As
;//    parameters, you pass the textures in which will get filled in with
;//    the resulting environment map.
;//
;// Face -
;//    Call this function to initiate the drawing of each face.  For each 
;//    environment map, you will call this six times.. once for each face 
;//    in D3DCUBEMAP_FACES.
;//
;// End -
;//    This will restore all render targets, and if needed compose all the
;//    rendered faces into the environment map surfaces.
;//
;// OnLostDevice, OnResetDevice -
;//    Call OnLostDevice() on this object before calling Reset() on the
;//    device, so that this object can release any stateblocks and video
;//    memory resources.  After Reset(), the call OnResetDevice().
;///////////////////////////////////////////////////////////////////////////

_D3DXRTE_DESC struct
	dx_Size			UINT ?
	MipLevels		UINT ?
 	Format			D3DFORMAT ?
	DepthStencil		BOOL ?
	DepthStencilFormat	D3DFORMAT ?
_D3DXRTE_DESC ends

D3DXRTE_DESC		typedef _D3DXRTE_DESC
LPD3DXRTE_DESC		typedef PTR _D3DXRTE_DESC

;// {1561135E-BC78-495b-8586-94EA537BD557}
IID_ID3DXRenderToEnvMap	TEXTEQU <{01561135eh,0bc78h,0495bh,{085h,086h,094h,0eah,053h,07bh,0d5h,057h}}>

_vtID3DXRenderToEnvMap MACRO CastName:REQ
	_vtIUnknown CastName
	&CastName&_GetDevice		comethod2 ?
	&CastName&_GetDesc		comethod2 ?
	&CastName&_BeginCube		comethod2 ?
	&CastName&_BeginSphere		comethod2 ?
	&CastName&_BeginHemisphere	comethod3 ?
	&CastName&_BeginParabolic	comethod3 ?
	&CastName&_Face			comethod3 ?
	&CastName&_End			comethod2 ?
	&CastName&_OnLostDevice		comethod1 ?
	&CastName&_OnResetDevice	comethod1 ?
ENDM

ID3DXRenderToEnvMap struct
	_vtID3DXRenderToEnvMap	ID3DXRenderToEnvMap
ID3DXRenderToEnvMap ends

LPD3DXRenderToEnvMap		typedef PTR DWORD

D3DXCreateRenderToEnvMap	PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD

;///////////////////////////////////////////////////////////////////////////
;// ID3DXLine:
;// ------------
;// This object intends to provide an easy way to draw lines using D3D.
;//
;// Begin - 
;//    Prepares device for drawing lines
;//
;// Draw -
;//    Draws a line strip in screen-space.
;//    Input is in the form of a array defining points on the line strip. of D3DXVECTOR2 
;//
;// DrawTransform -
;//    Draws a line in screen-space with a specified input transformation matrix.
;//
;// End - 
;//     Restores device state to how it was when Begin was called.
;//
;// SetPattern - 
;//     Applies a stipple pattern to the line.  Input is one 32-bit
;//     DWORD which describes the stipple pattern. 1 is opaque, 0 is
;//     transparent.
;//
;// SetPatternScale - 
;//     Stretches the stipple pattern in the u direction.  Input is one
;//     floating-point value.  0.0f is no scaling, whereas 1.0f doubles
;//     the length of the stipple pattern.
;//
;// SetWidth - 
;//     Specifies the thickness of the line in the v direction.  Input is
;//     one floating-point value.
;//
;// SetAntialias - 
;//     Toggles line antialiasing.  Input is a BOOL.
;//     TRUE  = Antialiasing on.
;//     FALSE = Antialiasing off.
;//
;// SetGLLines - 
;//     Toggles non-antialiased OpenGL line emulation.  Input is a BOOL.
;//     TRUE  = OpenGL line emulation on.
;//     FALSE = OpenGL line emulation off.
;//
;// OpenGL line:     Regular line:  
;//   *\                *\
;//   | \              /  \
;//   |  \            *\   \
;//   *\  \             \   \
;//     \  \             \   \
;//      \  *             \   *
;//       \ |              \ /
;//        \|               *
;//         *
;//
;// OnLostDevice, OnResetDevice -
;//    Call OnLostDevice() on this object before calling Reset() on the
;//    device, so that this object can release any stateblocks and video
;//    memory resources.  After Reset(), the call OnResetDevice().
;///////////////////////////////////////////////////////////////////////////



;// {72CE4D70-CC40-4143-A896-32E50AD2EF35}
IID_ID3DXLine		TEXTEQU	<{072ce4d70h,0cc40h,04143h,{0a8h,096h,032h,0e5h,00ah,0d2h,0efh,035h}}>

_vtID3DXLine MACRO CastName:REQ
	_vtIUnknown CastName
	&CastName&_GetDevice		comethod2 ?
	&CastName&_Begin		comethod1 ?
	&CastName&_Draw			comethod4 ?
	&CastName&_DrawTransform	comethod5 ?
	&CastName&_SetPattern		comethod2 ?
	&CastName&_GetPattern		comethod1 ?
	&CastName&_SetPatternScale	comethod2 ?
	&CastName&_GetPatternScale	comethod1 ?
	&CastName&_SetWidth		comethod2 ?
	&CastName&_GetWidth		comethod1 ?
	&CastName&_SetAntialias		comethod2 ?
	&CastName&_GetAntialias		comethod1 ?
	&CastName&_SetGLLines		comethod2 ?
	&CastName&_GetGLLines		comethod1 ?
	&CastName&_End			comethod1 ?
	&CastName&_OnLostDevice		comethod1 ?
	&CastName&_OnResetDevice	comethod1 ?
ENDM

ID3DXLine struct
	_vtID3DXLine	ID3DXLine
ID3DXLine ends

LPD3DXLINE		typedef PTR ID3DXLine

D3DXCreateLine		PROTO :DWORD,:DWORD

endif ;//__D3DX9CORE_H__
